/*-
 * $Id: rr-profile.c,v 1.12 2002/05/20 16:31:12 jonas Exp $
 *
 * See the file LICENSE for redistribution information. 
 * If you have not received a copy of the license, please contact CodeFactory
 * by email at info@codefactory.se, or on the web at http://www.codefactory.se/
 * You may also write to: CodeFactory AB, SE-903 47, Ume, Sweden.
 *
 * Copyright (c) 2002 Jonas Borgstrm <jonas@codefactory.se>
 * Copyright (c) 2002 Daniel Lundin   <daniel@codefactory.se>
 * Copyright (c) 2002 CodeFactory AB.  All rights reserved.
 */

#include <librr/rr.h>

static GObjectClass *parent_class = NULL;

static void finalize (GObject *object);

static void
rr_profile_init (GObject *object)
{
}

static void
rr_profile_class_init (GObjectClass *klass)
{
	klass->finalize = finalize;

	parent_class = g_type_class_peek_parent (klass);
}

GType 
rr_profile_get_type (void)
{
	static GType rr_type = 0;

	if (!rr_type) {
		static GTypeInfo type_info = {
			sizeof (RRProfileClass),
			NULL,
			NULL,
			(GClassInitFunc) rr_profile_class_init,
			NULL,
			NULL,
			sizeof (RRProfile),
			16,
			(GInstanceInitFunc) rr_profile_init
		};
		rr_type = g_type_register_static (RR_TYPE_CHANNEL, 
						  "RRProfile", 
						  &type_info, 
						  G_TYPE_FLAG_ABSTRACT);
	}
	return rr_type;
}

static void
finalize (GObject *object)
{
	RRProfile *profile = (RRProfile *)object;

	if (profile->piggyback)
		g_free (profile->piggyback);

	parent_class->finalize (object);
}

/**
 * rr_profile_get_uri:
 * @type: A GType
 * 
 * Get the URI the profile is associated with.
 * 
 * Return value: A uri or %NULL
 **/
const gchar *
rr_profile_get_uri (GType type)
{
	return g_type_get_qdata (type, 
				 g_quark_from_string (RR_PROFILE_GTYPE_KEY_URI));
}

/**
 * rr_profile_set_uri:
 * @type: A GType
 * @uri: An URI string.
 * 
 * Associate an URI with the profile.
 **/
void
rr_profile_set_uri (GType type, const gchar *uri)
{
	gchar *old;
	g_assert (g_type_is_a (type, RR_TYPE_PROFILE));

	old = g_type_get_qdata (type, 
				g_quark_from_string (RR_PROFILE_GTYPE_KEY_URI));
	if (old)
		g_free (old);

	g_type_set_qdata (type, 
			  g_quark_from_string (RR_PROFILE_GTYPE_KEY_URI), 
			  g_strdup (uri));
}

gboolean
rr_profile_validate_piggyback (RRProfile *profile, 
			       const gchar *piggyback,
			       gchar **reply)
{
	if (RR_PROFILE_GET_CLASS (profile)->validate_piggyback)
		return RR_PROFILE_GET_CLASS (profile)->validate_piggyback (profile, 
									   piggyback,
									   reply);
	else
		return TRUE;
}

void
rr_profile_reply_piggyback (RRProfile *profile, 
			    const gchar *piggyback)
{
	if (RR_PROFILE_GET_CLASS (profile)->reply_piggyback)
		RR_PROFILE_GET_CLASS (profile)->reply_piggyback (profile, 
								 piggyback);
}

gchar *
rr_profile_get_piggyback (RRProfile *profile)
{
	g_return_val_if_fail (RR_IS_PROFILE (profile), NULL);

	return profile->piggyback;
}

void
rr_profile_set_piggyback (RRProfile *profile, const gchar *piggyback)
{
	g_return_if_fail (RR_IS_PROFILE (profile));

	if (profile->piggyback)
		g_free (profile->piggyback);
	profile->piggyback = g_strdup (piggyback);
}
