/****************************************************************************/
/****************************************************************************/
/** \file packet.h
    \brief Header for the packet and protocol structures etc

    Header containing the typedef structs for the packet and protocol
    structures. Also contains function prototypes for building them. Actual
    functions are to be found in plugin-interface.c
*/
/****************************************************************************/

#include "types.h"
#include "protocols.h"

#ifndef PACKET_H
#define PACKET_H

/*! Each decoded protocol is placed in a linked list */
typedef struct PROTO_LL_S {
  PROTO protocol;          /*!< The name of the protocol */
  BYTE *hdr;               /*!< Pointer to the protocol header */
  ULONG hdr_len;           /*!< Length of the protocol header */
  BYTE *payload;           /*!< Pointer to the payload of the protocol */
  ULONG payload_len;       /*!< Length of the payload */
  BYTE *ftr;               /*!< Pointer to the footer */
  ULONG ftr_len;           /*!< Length of the footer */
  BOOL hasSpecials;        /*!< Does it have any optional etc headers */
  BOOL modified;           /*!< Whether the contents of this protocol (payload and header) have been modified */
  struct PROTO_LL_S *next;
} PROTO_LL;

/*! The main wrapper for all packets */
typedef struct PACKET_S {
  BYTE *packet;             /*!< Pointer to the start of the packet */
  ULONG packet_length;      /*!< Length of the entire packet */
  BYTE *payload;            /*!< Pointer to the highest decoded payload */
  ULONG payload_length;     /*!< Length of the payload */
  PROTO_LL *proto;          /*!< Pointer to linked list of decoded protocols*/
  PROTO nextProto;          /*!< Name of next proto if known */
  BOOL modified;            /*!< Whether any of the fields of the packet have been modified, hence requiring re-checksumming (possibly) */
} PACKET;

#endif /* PACKET_H */
