/****************************************************************************/
/****************************************************************************/
/** \file plugin.h
    \brief General plugin header

    Contains prototypes for many of the functions found in plugin-interface.c.
    It should be included by both plugins and core program files. Does not
    include any of the *plug.h, these should be done manually in the c files.
*/
/****************************************************************************/

#include "types.h"
#include "policy.h"
#include "packet.h"

#ifndef PLUGIN_H
#define PLUGIN_H

/*! Holder for input plugin information */
typedef struct INPLUG_S {
  char *name;               /*!< Name of plugin */
  long version;
} INPLUG;

/*! Holder for output plugin information */
typedef struct OUTPLUG_S {
  char *name;               /*!< Name of plugin */
  long version;
} OUTPLUG;

/*! Holder for analysis plugin information */
typedef struct ANPLUG_S {
  char *name;               /*!< Name of plugin */
  long version;
} ANPLUG;

/*! Holder for protocol plugin information */
typedef struct PROTOPLUG_S {
  char *name;               /*!< Name of plugin */
  unsigned short nProto;             /*!< Number of protocols supported */
  PROTO *pProto;            /*!< Array of nProto protocols */
  char **sProto;            /*!< Array of nProto protocol strings */
  long version;
} PROTOPLUG;

/*! Common holder for a plugin's information */
typedef struct PLUGIN_S {
  int (*pluginRegister)(INPLUG**, ANPLUG**, PROTOPLUG**, OUTPLUG**,
			int(*)(PACKET*,PROTO));
  int (*pluginInit)(int ct, char **inits);
  int (*pluginCleanup)(int reason);

  INPLUG *inPlug;
  int (*pluginGetPacket)(int iSrc, char *sSrc, PACKET **);

  OUTPLUG *outPlug;
  int (*pluginOutput)(PACKET *, char *args);

  ANPLUG *anPlug;
  char *(*pluginAnalyse)(PACKET*, char *anType, char *args);

  PROTOPLUG *protoPlug;
  int (*pluginDecode)(PACKET *, PROTO protocol);
  int (*pluginTest)(PACKET *, char *, char*);
  int (*pluginPrint)(PACKET *, char *args,
		  int(*print)(char *fmt, ...));
  int (*pluginMkTest)(PROTO, char*, char*, char *, POLICY_TEST *);

  struct PLUGIN_S *iNext;
  struct PLUGIN_S *aNext;
  struct PLUGIN_S *pNext;
  struct PLUGIN_S *oNext;
  struct PLUGIN_S *next;
} PLUGIN;


/* implemented in plugin-interface.c */
int loadPlugins(char *dir);
int initPlugin(PLUGIN *plug, int ct, char **inits);
int cleanupPlugins(void);
int decode(PACKET *, PROTO);
int testVar(PROTO proto, PACKET *p,
	    long offset, char varType, char testType, unsigned long val);
int testMask(PROTO proto, PACKET *p, 
	     long offset, long len, BYTE *val, BYTE *mask, BOOL type);
int testSpecial(PROTO proto, PACKET *p,
		char *test, char *val);
char *analyse(PROTO proto, PACKET *p, char*,char*);
int printVar(PROTO proto, PACKET *p,
	     char *args, int(*printFn)(char *fmt, ...));
int mkTest(PROTO proto, char *val, char *test, char *field, POLICY_TEST *pTest);

#endif /* PLUGIN_H */

