/****************************************************************************/
/****************************************************************************/
/** \file policy-dump.c
    \brief Dump the policy to a specific FILE*

*/
/****************************************************************************/

#include "policy-dump.h"
#include "memory.h"
#include <stdio.h>

void dumpOutput(FILE *fd, POLICY_OUTPUT *o, unsigned int tabs, char freeme){
  unsigned int i;
  if(!o) return;
  if(fd){
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    fprintf(fd, "OUTPUT(%s, %s", o->name, o->args);
    if(o->eof) fprintf(fd, ", EOF)\n");
    else fprintf(fd, ")\n");
    dumpTest(fd, o->tNext, tabs, freeme);
    dumpOutput(fd, o->oNext, tabs, freeme);
    if(freeme){
      free_safe(o->name);
      free_safe(o->args);
      free_safe(o);
    }
  }
  else if(freeme){
    dumpTest(fd, o->tNext, 0, freeme);
    dumpOutput(fd, o->oNext, tabs, freeme);
    free_safe(o->name);
    free_safe(o->args);
    free_safe(o);
  }
}

void dumpProto(FILE *fd, POLICY_PROTO *p, unsigned int tabs, char freeme){
  unsigned int i;
  if(!p) return;
  if(fd){
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    fprintf(fd, "PROTO(%lx, %s)\n", p->proto_num, p->proto_name);
    dumpTest(fd, p->test, tabs+2, freeme);
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    fprintf(fd, "END_PROTO\n");
    dumpProto(fd, p->next, tabs, freeme);
    if(freeme){
      free_safe(p->proto_name);
      free_safe(p);
    }
  }
  else if(freeme){
    dumpTest(fd, p->test, 0, freeme);
    dumpProto(fd, p->next, 0, freeme);
    free_safe(p->proto_name);
    free_safe(p);
  }
}

void dumpTest(FILE *fd, POLICY_TEST *t, unsigned int tabs, char freeme){
  unsigned int i;
  unsigned long ul;

  if(!t) return;
  if(fd){
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    if(t->test_type==POLICY_TEST_TYPE_MASKED){
      fprintf(fd, "TEST(p_name=%s,p_num=%lx,v=",t->p_name, t->p_num);
      for(ul=0;ul<t->m_len;ul++){
	fprintf(fd, "%2.2hhx",(t->m_val)[ul]);
      }
      fprintf(fd, ",m=");
      for(ul=0;ul<t->m_len;ul++){
	fprintf(fd, "%2.2hhx",(t->m_mask)[ul]);
      }
      fprintf(fd, ",o=%lx,l=%lx,t=%c",t->m_offset,t->m_len,t->m_type);
    }
    else if(t->test_type==POLICY_TEST_TYPE_VAR){
      fprintf(fd, "TEST(p_name=%s,p_num=%lx,v=%lx,vt=%c,tt=%c%c",t->p_name,t->p_num,t->v_val,t->v_vType,t->v_tType[0],t->v_tType[1]);
    }
    else if(t->test_type==POLICY_TEST_TYPE_ANALYSIS){
      fprintf(fd, "ANALYSIS(%s, %s", t->a_name, t->a_args);
    }
    else {
      fprintf(fd, "TEST - UNKNOWN TYPE ");
    }
    fprintf(fd, ",p_num=%lx,p_name=%s)\n",t->p_num,t->p_name);
    dumpResult(fd, t->result, tabs+2, freeme);
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    if(t->test_type==POLICY_TEST_TYPE_ANALYSIS){
      fprintf(fd, "END_ANALYSIS\n");
    }
    else {
      fprintf(fd, "END_TEST\n");
    }
    if(freeme){
      if(t->test_type==POLICY_TEST_TYPE_MASKED){
	free_safe(t->m_val);
	free_safe(t->m_mask);
      }
      free_safe(t->p_name);
      free_safe(t);
    }
  }
  else if(freeme){
    dumpResult(fd, t->result, 0, freeme);
    if(t->test_type==POLICY_TEST_TYPE_MASKED){
      free_safe(t->m_val);
      free_safe(t->m_mask);
    }
    free_safe(t->p_name);
    free_safe(t);
  }
}

void dumpResult(FILE *fd, POLICY_RESULT *r, unsigned int tabs, char freeme){
  unsigned int i;
  if(!r) return;
  if(fd){
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    fprintf(fd, "RESULT(%s,%hu)\n",r->sResult,r->iResult);
    dumpOutput(fd, r->match, tabs+2, freeme);
    for(i=0;i<tabs;i++){
      fprintf(fd, ".");
    }
    fprintf(fd, "END_RESULT\n");
    dumpResult(fd, r->next, tabs, freeme);
    if(freeme){
      free_safe(r->sResult);
      free_safe(r);
    }
  }
  else if(freeme){
    dumpOutput(fd, r->match, 0, freeme);
    dumpResult(fd, r->next, tabs, freeme);
    free_safe(r->sResult);
    free_safe(r);
  }
}

void dumpInit(FILE *fd, POLICY_INITS *i, unsigned int tabs, char freeme){
  unsigned int u;
  if(!i) return;
  if(fd){
    for(u=0;u<tabs;u++){
      fprintf(fd, ".");
    }
    fprintf(fd, "INIT(%s",i->plugname);
    for(u=0;u<i->string_count;u++){
      fprintf(fd, ",%s",(i->init_string)[u]);
    }
    fprintf(fd, ")\n");
    dumpInit(fd, i->next, tabs, freeme);  
    if(freeme){
      for(u=0;u<i->string_count;u++){
	free_safe((i->init_string)[u]);
      }
      free_safe(i);
    }
  }
  else if(freeme){
    dumpInit(fd, i->next, 0, freeme);
    for(u=0;u<i->string_count;u++){
      free_safe((i->init_string)[u]);
    }
    free_safe(i);
  }
}

void dumpInput(FILE *fd, POLICY_INPUT *i, unsigned int tabs, char freeme){
  unsigned int u;
  if(!i) return;
  if(fd){
    for(u=0;u<tabs;u++){
      fprintf(fd, ".");
    }
    fprintf(fd, "INPUT(%s,%hu,%s,%u)\n", i->plugname, i->weight, i->sSrc, i->iSrc);
    dumpInput(fd, i->next, tabs, freeme);
    if(freeme){
      free_safe(i->plugname);
      free_safe(i->sSrc);
      free_safe(i);
    }
  }
  else if(freeme){
    dumpInput(fd, i->next, 0, freeme);
    free_safe(i->plugname);
    free_safe(i->sSrc);
    free_safe(i);
  }
}

void dumpPolicy(FILE *fd, POLICY_HEAD *h, char freeme){
  if(!h) return;
  if(fd){
    fprintf(fd, "POLICY");
    fprintf(fd, "..INITS\n");
    dumpInit(fd, h->inits, 4, freeme);
    fprintf(fd, "..END_INITS\n..INPUTS\n");
    dumpInput(fd, h->input, 4, freeme);
    fprintf(fd, "..END_INPUTS\n..DEFAULTS\n");
    dumpOutput(fd, h->def, 4, freeme);
    fprintf(fd, "..END_DEFAULTS\n..PROTOS\n");
    dumpProto(fd, h->top, 4, freeme);
    fprintf(fd, "..END_PROTOS\nEND_POLICY\n");
    if(freeme) free_safe(h);
  }
  else if(freeme){
    dumpInit(fd, h->inits, 0, freeme);
    dumpInput(fd, h->input, 0, freeme);
    dumpOutput(fd, h->def, 0, freeme);
    dumpProto(fd, h->top, 0, freeme);
    free_safe(h);
  }
}
