/****************************************************************************/
/****************************************************************************/
/** \file policy.h
    \brief Header for the policy thread

*/
/****************************************************************************/

#include "types.h"

#ifndef POLICY_H
#define POLICY_H

struct POLICY_OUTPUT_S;
struct POLICY_PROTO_S;
struct POLICY_TEST_S;
struct POLICY_RESULT_S;
struct POLICY_INITS_S;
struct POLICY_INPUT_S;

typedef struct POLICY_HEAD_S {
  struct POLICY_PROTO_S *top;
  struct POLICY_OUTPUT_S *def;
  struct POLICY_INITS_S *inits;
  struct POLICY_INPUT_S *input;
} POLICY_HEAD;

typedef struct POLICY_INPUT_S {
  char *plugname;
  unsigned short weight;
  char *sSrc;
  int iSrc;
  struct POLICY_INPUT_S *next;
} POLICY_INPUT;

typedef struct POLICY_INITS_S {
  char *plugname;
  char **init_string;
  unsigned int string_count;
  unsigned int space;
  struct POLICY_INITS_S *next;
} POLICY_INITS;

typedef struct POLICY_PROTO_S {
  PROTO proto_num;
  char *proto_name;
  struct POLICY_TEST_S  *test;
  struct POLICY_PROTO_S *next;
} POLICY_PROTO;

#define POLICY_TEST_TYPE_MASKED   0x01
#define POLICY_TEST_TYPE_VAR      0x02
#define POLICY_TEST_TYPE_ANALYSIS 0x03

#define POLICY_TEST_V_TYPE_8   'b'
#define POLICY_TEST_V_TYPE_U8  'B'
#define POLICY_TEST_V_TYPE_16  's'
#define POLICY_TEST_V_TYPE_U16 'S'
#define POLICY_TEST_V_TYPE_32  'l'
#define POLICY_TEST_V_TYPE_U32 'L'

typedef struct POLICY_TEST_S {
  PROTO p_num;
  char *p_name;
  char test_type;
  /* masked test */
  BYTE *m_val;
  BYTE *m_mask;
  unsigned long m_offset;
  unsigned long m_len;
  char m_type; /* = or ! */
  /* var test */
  unsigned long v_val;
  char v_vType;    /* see the variable types above */
  char *v_tType; /* != == < > >= =< */ 
  unsigned long v_offset;
  /* analysis test */
  char *a_name;
  char *a_args;
  /* pointers */
  struct POLICY_RESULT_S *result;
} POLICY_TEST;

typedef struct POLICY_RESULT_S {
  char *sResult;
  unsigned short iResult;
  struct POLICY_RESULT_S *next;
  struct POLICY_OUTPUT_S *match;
} POLICY_RESULT;

typedef struct POLICY_OUTPUT_S {
  char *name;
  char *args;
  char eof;
  struct POLICY_OUTPUT_S *oNext;
  struct POLICY_TEST_S *tNext;
} POLICY_OUTPUT;





#endif /* POLICY_H */
